#!/usr/bin/env python
import os
import subprocess
import sys
import tarfile
import shlex
import shutil
import inspect

from hdputils import install_base_pkg, updateHDPDirectoryScript, config_all_nodes, \
                    setHadoopEnvVar, home, hdpScript, configureJAVA, config_all_nodes
from bdutils import setDirPermission, fileSetKV, append_bashrc, is_jvm_service_active, setHadoopConfigXML, chownRecursive

from charmhelpers.lib.utils import config_get
from shutil import rmtree, copyfile
from socket import gethostname
from charmhelpers.core.hookenv import log, Hooks, relation_get, relation_set, unit_get, open_port
from charmhelpers.core.host import service_start, service_stop, add_user_to_group
from time import sleep

# Required for unit tests... :(
try:
    from charmhelpers.fetch import apt_install, apt_update,  apt_purge
except:
    pass

               
# Parses /proc/meminfo for total memory in GB
def get_system_total_memory():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    with open("/proc/meminfo") as f:
        lines = f.readlines()
        for i in range(0, len(lines)):
            if lines[i].startswith("MemTotal"):
                n = int (lines[i].split()[1])
                return str(n/1000000)      
   
# Parses /proc/cpuinfo for number of physical cores          
def get_system_cores():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    with open("/proc/cpuinfo") as f:
        lines = f.readlines()
        for i in range(0, len(lines)):
            if lines[i].startswith("cpu cores"):
                return str(lines[i].split()[3])
            
def get_system_disk():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    #TODO must get from config.yaml
    return str(1)

def is_hbase_installed():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    #TODO must get from config.yaml
    return "False"
         
   
def calculateHDPOptMemConfSetting():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    os.chdir(hdpScriptPath)
    cmd = "python hdp-configuration-utils.py -c "+get_system_cores()+" -m "+\
           get_system_total_memory()+" -d "+get_system_disk()+" -k "+is_hbase_installed()+\
           " >"+hadoopMemoryOptimizationData
    log (" cmd ="+cmd, "info")
    subprocess.check_output(cmd.split())
            
# Update HDP directories.sh file with user defined configuration defined
# in config.yaml
def updateDirectoriesScript():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    fileSetKV(directoriesScript, "DFS_NAME_DIR=", "\""+config_get('dfs_name_dir')+"\";")
    fileSetKV(directoriesScript, "DFS_DATA_DIR=","\""+ config_get('dfs_data_dir')+"\";")
    fileSetKV(directoriesScript, "FS_CHECKPOINT_DIR=", "\""+config_get('fs_checkpoint_dir')+"\";")
    fileSetKV(directoriesScript, "YARN_LOCAL_DIR=", "\""+config_get('yarn_local_dir')+"\";")
    fileSetKV(directoriesScript, "YARN_LOCAL_LOG_DIR=", "\""+config_get('yarn_local_log_dir')+"\";")
    fileSetKV(directoriesScript, "ZOOKEEPER_DATA_DIR=", "\""+config_get('zookeeper_data_dir')+"\";") 
    

def createHDPHadoopConf():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
   
    HADOOP_CONF_DIR = os.environ["HADOOP_CONF_DIR"]
    HDPConfPath = os.path.join(os.path.sep,home, hdpScript, "configuration_files", "core_hadoop")
    source = os.listdir(HDPConfPath)
    for files in source:
        srcFile = os.path.join(os.path.sep, HDPConfPath, files)
        desFile = os.path.join(os.path.sep, HADOOP_CONF_DIR, files)
        shutil.copyfile(srcFile, desFile)
    subprocess.call(createDadoopConfDir)
     
     
    
def uninstall_base_pkg():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    packages = ['ntp',
                'openjdk-7-jdk',
                'hadoop',
                'hadoop-hdfs', 
                'hadoop-yarn',
                'hadoop-mapreduce',
                'hadoop-client',
                'openssl',
                'libsnappy1',
                'libsnappy-dev',
                'liblzo2-2',
                'liblzo2-dev',
                'libhdfs0',
                'libhdfs0-dev',
                'hadoop-lzo']
    apt_purge(packages)
    shutil.rmtree(hdpScriptPath)
    shutil.copyfile(bashrc+".org", bashrc)
 
   
def config_hadoop_nodes():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    updateDirectoriesScript()
    config_all_nodes()
    setHadoopEnvVar()
    log("==> calling subprocess.call(hdp_hellper_script)","INFO")
    subprocess.call(hdp_hellper_script)
    calculateHDPOptMemConfSetting()
    createHDPHadoopConf()
    group = os.environ['HADOOP_GROUP']
    add_user_to_group("ubuntu", group)
    add_user_to_group("ubuntu", "mapred")
    setDirPermission(os.environ['HDFS_LOG_DIR'], os.environ['HDFS_USER'], group, 0755)
    setDirPermission(os.environ['YARN_LOG_DIR'], os.environ['YARN_USER'], group, 0755)
    setDirPermission(os.environ['HDFS_PID_DIR'], os.environ['HDFS_USER'], group, 0755)
    setDirPermission(os.environ['MAPRED_LOG_DIR'], os.environ['MAPRED_USER'], group, 0755)
    setDirPermission(os.environ['MAPRED_PID_DIR'], os.environ['MAPRED_USER'], group, 0755)
    setDirPermission(os.environ['YARN_LOCAL_DIR'], os.environ['YARN_USER'], group, 0755)
    setDirPermission(os.environ['YARN_LOCAL_LOG_DIR'], os.environ['YARN_USER'], group, 0755)
    hdfsConfPath = os.path.join(os.path.sep, os.environ['HADOOP_CONF_DIR'],'hdfs-site.xml')
    setHadoopConfigXML(hdfsConfPath, "dfs.namenode.name.dir", config_get('dfs_name_dir'))
    setHadoopConfigXML(hdfsConfPath, "dfs.datanode.data.dir", config_get('dfs_data_dir'))
    setHadoopConfigXML(hdfsConfPath, "dfs.namenode.checkpoint.dir", config_get('fs_checkpoint_dir'))
    
# candidate for BD charm helper     
def format_namenode(hdfsUser):
    log("==> hdfs format for user={}".format(hdfsUser),"INFO")
    cmd = shlex.split("su {} -c '/usr/lib/hadoop/bin/hadoop namenode -format'".format(hdfsUser))
    subprocess.call(cmd)
    
def callHDFS_fs(command):
    cmd = shlex.split("su hdfs -c '/usr/lib/hadoop/bin/hadoop fs {}'".format(command))
    subprocess.call(cmd)
    
def startJobHistory():
    log("==> startJobHistory")
    path = os.path.join(os.path.sep, 'usr', 'lib', 'hadoop-yarn', 'bin', 'container-executor')
    chownRecursive(path, 'root', 'hadoop')
    os.chmod(path, 650)
    callHDFS_fs("-mkdir -p /mr-history/tmp")
    callHDFS_fs("-chmod -R 1777 /mr-history/tmp")
    callHDFS_fs("-mkdir -p /mr-history/done")
    callHDFS_fs("-chmod -R 1777 /mr-history/done")
    callHDFS_fs("-chown -R {}:{} /mr-history".format(os.environ['MAPRED_USER'], os.environ['HDFS_USER']))
    callHDFS_fs("-mkdir -p /app-logs")
    callHDFS_fs("-chmod -R 1777 /app-logs ")
    callHDFS_fs("-chown yarn /app-logs ")
    hadoopConfDir = os.environ["HADOOP_CONF_DIR"]
    os.environ["HADOOP_LIBEXEC_DIR"]="/usr/lib/hadoop/libexec"
    cmd = shlex.split("su {} -c '/usr/lib/hadoop-mapreduce/sbin/mr-jobhistory-daemon.sh --config {} start historyserver'".\
                      format(os.environ['MAPRED_USER'], hadoopConfDir))
    subprocess.call(cmd)
    
def stopJobHistory():
    hadoopConfDir = os.environ["HADOOP_CONF_DIR"]
    os.environ["HADOOP_LIBEXEC_DIR"]="/usr/lib/hadoop/libexec"
    cmd = shlex.split("su {} -c '/usr/lib/hadoop-yarn/sbin/yarn-daemon.sh --config {} stop resourcemanager'".\
                      format(os.environ['MAPRED_USER'], hadoopConfDir))
    subprocess.call(cmd)

# candidate for BD charm helper       

def start_namenode(hdfsUser):
    log("==> start namenode for user={}".format(hdfsUser), "INFO")
    hadoopConfDir = os.environ["HADOOP_CONF_DIR"]
    cmd = shlex.split("su {} -c '/usr/lib/hadoop/sbin/hadoop-daemon.sh --config {} start namenode'".\
                      format(hdfsUser, hadoopConfDir))
    subprocess.call(cmd)
    
def stop_namenode(hdfsUser):
    log("==> start namenode for user={}".format(hdfsUser), "INFO")
    hadoopConfDir = os.environ["HADOOP_CONF_DIR"]
    cmd = shlex.split("su {} -c '/usr/lib/hadoop/sbin/hadoop-daemon.sh --config {} stop namenode'".\
                      format(hdfsUser, hadoopConfDir))
    subprocess.call(cmd)
# candidate for BD charm helper

def start_datanode(hdfsUser):
    log("==> start namenode for user={}".format(hdfsUser), "INFO")
    hadoopConfDir = os.environ["HADOOP_CONF_DIR"]
    cmd = shlex.split("su {} -c '/usr/lib/hadoop/sbin/hadoop-daemon.sh --config {} start datanode'".\
                      format(hdfsUser, hadoopConfDir))
    subprocess.call(cmd)
    
def stop_datanode(hdfsUser):
    log("==> start namenode for user={}".format(hdfsUser), "INFO")
    hadoopConfDir = os.environ["HADOOP_CONF_DIR"]
    cmd = shlex.split("su {} -c '/usr/lib/hadoop/sbin/hadoop-daemon.sh --config {} stop datanode'".\
                      format(hdfsUser, hadoopConfDir))
    subprocess.call(cmd)
# candidate for BD charm helper
def configureYarn(RMhostname):
    yarnConfPath = os.path.join(os.path.sep, os.environ['HADOOP_CONF_DIR'],"yarn-site.xml")
    mapConfDir = os.path.join(os.path.sep, os.environ['HADOOP_CONF_DIR'],"mapred-site.xml")
    
    setHadoopConfigXML(yarnConfPath, "yarn.resourcemanager.scheduler.class",\
                       "org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.CapacityScheduler")
    setHadoopConfigXML(yarnConfPath, "yarn.resourcemanager.resource-tracker.address", RMhostname+":8025")
    setHadoopConfigXML(yarnConfPath, "yarn.resourcemanager.scheduler.address", RMhostname+":8030")
    setHadoopConfigXML(yarnConfPath, "yarn.resourcemanager.address", RMhostname+":8050")
    setHadoopConfigXML(yarnConfPath, "yarn.resourcemanager.admin.address", RMhostname+":8141")
    setHadoopConfigXML(yarnConfPath, "yarn.nodemanager.local-dirs", config_get('yarn_local_dir'))
    setHadoopConfigXML(yarnConfPath, "yarn.nodemanager.log-dirs", config_get('yarn_local_log_dir'))
    setHadoopConfigXML(yarnConfPath, "yarn.log.server.url","http://"+RMhostname+":19888")
    setHadoopConfigXML(yarnConfPath, "yarn.resourcemanager.webapp.address", RMhostname+":8088")
    #jobhistory server
    setHadoopConfigXML(mapConfDir, "mapreduce.jobhistory.webapp.address", RMhostname+":19888")
    setHadoopConfigXML(mapConfDir, "mapreduce.jobhistory.address",  RMhostname+":10020")
    
# candidate for BD charm helper    
def start_RM(yarnUser):
    log("==> start resourcemanager", "INFO")
    hadoopConfDir = os.environ["HADOOP_CONF_DIR"]
    os.environ["HADOOP_LIBEXEC_DIR"]="/usr/lib/hadoop/libexec"
    cmd = shlex.split("su {} -c '/usr/lib/hadoop-yarn/sbin/yarn-daemon.sh --config {} start resourcemanager'".\
                      format(yarnUser, hadoopConfDir))
    subprocess.call(cmd)
    
def stop_RM(yarnUser):
    log("==> stop resourcemanager", "INFO")
    hadoopConfDir = os.environ["HADOOP_CONF_DIR"]
    os.environ["HADOOP_LIBEXEC_DIR"]="/usr/lib/hadoop/libexec"
    cmd = shlex.split("su {} -c '/usr/lib/hadoop-yarn/sbin/yarn-daemon.sh --config {} stop resourcemanager'".\
                      format(yarnUser, hadoopConfDir))
    subprocess.call(cmd)
# candidate for BD charm helper
def start_NM(yarnUser):
    log("==> start nodemanager", "INFO")
    hadoopConfDir = os.environ["HADOOP_CONF_DIR"]
    os.environ["HADOOP_LIBEXEC_DIR"]="/usr/lib/hadoop/libexec"
    cmd = shlex.split("su {} -c '/usr/lib/hadoop-yarn/sbin/yarn-daemon.sh --config {} start nodemanager'".\
                      format(yarnUser, hadoopConfDir))
    subprocess.call(cmd)
    
def stop_NM(yarnUser):
    log("==> stop nodemanager", "INFO")
    hadoopConfDir = os.environ["HADOOP_CONF_DIR"]
    os.environ["HADOOP_LIBEXEC_DIR"]="/usr/lib/hadoop/libexec"
    cmd = shlex.split("su {} -c '/usr/lib/hadoop-yarn/sbin/yarn-daemon.sh --config {} stop nodemanager'".\
                      format(yarnUser, hadoopConfDir))
    subprocess.call(cmd)
    
def stop_hadoop_services():
    if is_jvm_service_active("ResourceManager"):
        stop_RM(os.environ['YARN_USER'])
    if is_jvm_service_active("NodeManager"):
        stop_NM(os.environ['YARN_USER'])
    if is_jvm_service_active("NameNode"):
        stop_namenode(os.environ['HDFS_USER'])
    if is_jvm_service_active("DataNode"):
        stop_datanode(os.environ['HDFS_USER'])
        
def restart_hadoop_services():
    if is_jvm_service_active("ResourceManager"):
        stop_RM(os.environ['YARN_USER'])
        start_RM(os.environ['YARN_USER'])
    if is_jvm_service_active("NodeManager"):
        stop_NM(os.environ['YARN_USER'])
        start_NM(os.environ['YARN_USER'])
    if is_jvm_service_active("NameNode"):
        stop_namenode(os.environ['HDFS_USER'])
        start_namenode(os.environ['HDFS_USER'])
    if is_jvm_service_active("DataNode"):
        stop_datanode(os.environ['HDFS_USER'])
        start_datanode(os.environ['HDFS_USER'])
        
def configureHDFS(hostname):
    hdfsConfPath = os.path.join(os.path.sep, os.environ['HADOOP_CONF_DIR'],'hdfs-site.xml')
    coreConfPath = os.path.join(os.path.sep, os.environ['HADOOP_CONF_DIR'],'core-site.xml')
    setHadoopConfigXML(coreConfPath, "fs.defaultFS", "hdfs://"+hostname+":8020")
    setHadoopConfigXML(hdfsConfPath, "dfs.namenode.http-address", hostname+":50070")
        
#################################### Global Data ################################     


hdp_hellper_script = os.path.join(os.path.sep, os.environ['CHARM_DIR'],'files', 'scripts', "HDPHelperScripts.sh")
if not os.path.isfile(hdp_hellper_script):
    log ("Erro ==> {} not found".format(hdp_hellper_script), "ERROR")
    

createDadoopConfDir = os.path.join(os.path.sep, os.environ['CHARM_DIR'],'files', 'scripts', "createHadoopConfDir.sh")
hdpScript = "hdp_scripts"
hdpScriptPath = os.path.join(os.path.sep,home, hdpScript,'scripts')
usersAndGroupsScript = os.path.join(os.path.sep, hdpScriptPath, "usersAndGroups.sh")
directoriesScript =  os.path.join(os.path.sep, hdpScriptPath, "directories.sh")
tarfilename="hdp_manual_install_rpm_helper_files-2.1.1.385.tar.gz"
tarfilenamePre="hdp_manual_install_rpm_helper_files-2.1.1.385"
bashrc = os.path.join(os.path.sep, home, '.bashrc')
hadoopMemoryOptimizationData = os.path.join(os.path.sep, hdpScriptPath, "hdpMemOpt.txt");
nodeType="unknown"
nameNodeReady = False
resourceManagerReady = False
##########################################################################################



# Start the Hook Logic Block
hooks = Hooks()


@hooks.hook('install')
def install():
    packages = ['ntp',
                'openjdk-7-jdk',
                'hadoop',
                'hadoop-hdfs',  
                'hadoop-yarn',
                'hadoop-mapreduce',
                'hadoop-client',
                'openssl',
                'libsnappy1',
                'libsnappy-dev',
                'liblzo2-2',
                'liblzo2-dev',
                'libhdfs0',
                'libhdfs0-dev',
                'hadoop-lzo']
    install_base_pkg(packages)
    config_hadoop_nodes()
        
    
@hooks.hook('resourcemanager-relation-joined')
def resourcemanager_relation_joined():
    log ("==> resourcemanager-relation-joined","INFO")
    if is_jvm_service_active("ResourceManager"):
        relation_set(resourceManagerReady=True)
        relation_set(resourceManager_hostname=gethostname())
        return
    setHadoopEnvVar()
    nodeType="resourcemanager"
    relation_set(resourceManager_hostname=gethostname())
    configureYarn(gethostname())
    start_RM(os.environ["YARN_USER"])
    startJobHistory()
    open_port(8025)
    open_port(8030)
    open_port(8050)
    open_port(8141)
    open_port(19888)
    open_port(8088)
    open_port(10020)
    sleep(5)
    relation_set(resourceManagerReady=True)

@hooks.hook('nodemanager-relation-changed')
def nodemanager_relation_changed():
    log ("nodemanager_relation_changed","INFO")
    resourceManagerReady  = relation_get("resourceManagerReady")
    if not resourceManagerReady:
        sys.exit(0)
    setHadoopEnvVar()
    configureYarn(relation_get('resourceManager_hostname'))
    start_NM(os.environ["YARN_USER"])
    open_port(8025)
    open_port(8030)
    open_port(8050)
    open_port(8141)
    open_port(19888)
    open_port(8088)
    open_port(10020)
    
          
@hooks.hook('resourcemanager-relation-broken')
def resourcemanager_relation_broken():
    log ("Configuring resourcemanager - broken phase","INFO")
    setHadoopEnvVar()
    
@hooks.hook('resourcemanager-relation-changed')
def resourcemanager_relation_changed():
    log ("Configuring resourcemanager - changed phase","INFO")
    setHadoopEnvVar()      

@hooks.hook('namenode-relation-joined')
def namenode_relation_joined():
    log("Configuring namenode - joined phase", "INFO")
    
    if is_jvm_service_active("NameNode"):
        relation_set(nameNodeReady=True)
        relation_set(namenode_hostname=gethostname())
        return
    setHadoopEnvVar()
    nodeType="namenode"
    setDirPermission(os.environ['DFS_NAME_DIR'], os.environ['HDFS_USER'], os.environ['HADOOP_GROUP'], 0755)
    relation_set(namenode_hostname=gethostname())
    configureHDFS(gethostname())
    format_namenode(os.environ["HDFS_USER"])
    start_namenode(os.environ["HDFS_USER"])
    
    open_port(8020)
    open_port(50070)
    open_port(8480)
    open_port(50470)
    sleep(5)
    relation_set(nameNodeReady=True)
    
    

@hooks.hook('datanode-relation-changed')
def datanode_relation_changed():
    log("Configuring datanode - Changed phase","INFO")
    nameNodeReady  = relation_get("nameNodeReady")
    if not nameNodeReady:
        sys.exit(0)
    setHadoopEnvVar()
    nodeType="namenode"
    namenode_hostname = relation_get("namenode_hostname")
    configureHDFS(namenode_hostname)
    setDirPermission(os.environ['DFS_DATA_DIR'], os.environ['HDFS_USER'], os.environ['HADOOP_GROUP'], 0750)
    start_datanode(os.environ["HDFS_USER"])
    open_port(8010)
    open_port(8480)
    open_port(50010)
    open_port(50075)
    
        
@hooks.hook('config-changed')
def config_changed():     
    log( "config-changed called", "INFO")  
    setHadoopEnvVar()   
    #config_hadoop_nodes
    restart_hadoop_services
    
    

@hooks.hook('namenode-relation-changed')
def namenode_relation_changed():
    log("Configuring namenode - changed phase", "INFO")  

@hooks.hook('start')
def start():     
    log( "start called", "INFO")
      
@hooks.hook('stop')
def stop():     
    log( "stop called", "INFO")
    setHadoopEnvVar()
    stop_hadoop_services()
    
    
    

if __name__ == "__main__":
    hooks.execute(sys.argv)