#!/usr/bin/python3

import amulet
import yaml

class TestDeployment(object):
    def __init__(self):
        self.d =  amulet.Deployment(series='trusty')
        f = open('hadoop_cluster.yaml')
        bun = f.read()
        self.d.load(yaml.safe_load(bun))
        try:
            self.d.setup(timeout=9000)
            self.d.sentry.wait()
        except amulet.helpers.TimeoutError:
            amulet.raise_status(amulet.SKIP, msg="Environment wasn't stood up in time")
        except:
            raise
        self.master_unit = self.d.sentry.unit['yarn-hdfs-master/0']
        self.compute_unit = self.d.sentry.unit['compute-node/0']
   
    def run(self):
        for test in dir(self):
            if test.startswith('test_'):
                getattr(self, test)()
############################################################
# Validate hadoop services on master node have been started
############################################################
    def test_hadoop_master_service_status(self):
        o,c= self.master_unit.run("jps | awk '{print $2}'")
        if o.find('ResourceManager') == -1:
            amulet.raise_status(amulet.FAIL, msg="ResourceManager not started")
        else:
            amulet.raise_status(amulet.PASS, msg="ResourceManager started")
        if o.find('JobHistoryServer') == -1:
            amulet.raise_status(amulet.FAIL, msg="JobHistoryServer not started")
        else:
            amulet.raise_status(amulet.PASS, msg="JobHistoryServer started")
        if o.find('NameNode') == -1:
            amulet.raise_status(amulet.FAIL, msg="NameNode not started")
        else:
            amulet.raise_status(amulet.PASS, msg="NameNode started")
            
############################################################
# Validate hadoop services on compute node have been started
############################################################
    def test_hadoop_compute_service_status(self):
        o,c= self.compute_unit.run("jps | awk '{print $2}'")
        if o.find('NodeManager') == -1:
            amulet.raise_status(amulet.FAIL, msg="NodeManager not started")
        else:
            amulet.raise_status(amulet.PASS, msg="NodeManager started")
        if o.find('DataNode') == -1:
            amulet.raise_status(amulet.FAIL, msg="DataServer not started")
        else:
            amulet.raise_status(amulet.PASS, msg="DataServer started")
###########################################################################
# Validate admin few hadoop activities on HDFS cluster.
###########################################################################
# 1) This test validates mkdir on hdfs cluster
###########################################################################
    def test_hdfs_mkdir(self):
        o,c= slef.master_unit.run("su hdfs -c 'hdfs dfs -mkdir -p /user/ubuntu'")
        if c == 0:
            amulet.raise_status(amulet.PASS, msg=" Successfully created a user directory on hdfs")
        else:
            amulet.raise_status(amulet.FAIL, msg=" Created a user directory on hdfs FAILED")
###########################################################################
# 2) This test validates change hdfs dir owner on the cluster
###########################################################################
    def test_hdfs_dir_owner(self):
        o,c= self.master_unit.run("su hdfs -c 'hdfs dfs -chown ubuntu:ubuntu /user/ubuntu'")
        if c == 0:
            amulet.raise_status(amulet.PASS, msg=" Successfully assigned an owner to directory on hdfs")
        else:
            amulet.raise_status(amulet.FAIL, msg=" assigning an owner to hdfs directory FAILED")
            
###########################################################################
# 3) This test validates setting hdfs directory access permission on the cluster
###########################################################################
    def test_hdfs_dir_permission(self):
        o,c= self.master_unit.run("su hdfs -c 'hdfs dfs -chmod -R 755 /user/ubuntu'")
        if c == 0:
            amulet.raise_status(amulet.PASS, msg=" Successfully set directory permission on hdfs")
        else:
            amulet.raise_status(amulet.FAIL, msg=" seting directory permission on hdfs FAILED")
   
###########################################################################
# Validate yarn mapreduce operation
# 1) validate mapreduce execution - writing to hdfs
# 2) validate successful mapreduce operation after the execution
###########################################################################
    def test_yarn_mapreduce_exe1(self):
        o,c= self.master_unit.run("su ubuntu -c 'hadoop jar /usr/lib/hadoop-mapreduce/hadoop-mapreduce-examples-*.jar teragen  10000 /user/ubuntu/teragenout'")
        if c == 0:
            amulet.raise_status(amulet.PASS, msg=" Successfull execution of teragen mapreduce app")
        else:
            amulet.raise_status(amulet.FAIL, msg=" teragen FAILED")   
###########################################################################
# 2) validate successful mapreduce operation after the execution
###########################################################################
        o,c= self.master_unit.run("su hdfs -c 'hdfs dfs -ls /user/ubuntu/teragenout/_SUCCESS'")
        if c == 0:
            amulet.raise_status(amulet.PASS, msg="mapreduce operation was success")
        else:
            amulet.raise_status(amulet.FAIL, msg="mapreduce operation was not a success")

###########################################################################
# Validate yarn mapreduce operation
# 1) validate mapreduce execution - reading and writing to hdfs
# 2) validate successful mapreduce operation after the execution
###########################################################################
    def test_yarn_mapreduce_exe2(self):
        o,c= self.master_unit.run("su ubuntu -c 'hadoop jar /usr/lib/hadoop-mapreduce/hadoop-mapreduce-examples-*.jar terasort /user/ubuntu/teragenout /user/ubuntu/terasortout'")
        if c == 0:
            amulet.raise_status(amulet.PASS, msg=" Successfull execution of terasort mapreduce app")
        else:
            amulet.raise_status(amulet.FAIL, msg=" terasort FAILED")
###########################################################################
# 2) validate a successful mapreduce operation after the execution
###########################################################################
        o,c= self.master_unit.run("su hdfs -c 'hdfs dfs -ls /user/ubuntu/terasortout/_SUCCESS'")
        if c == 0:
            amulet.raise_status(amulet.PASS, msg="mapreduce operation was success")
        else:
            amulet.raise_status(amulet.FAIL, msg="mapreduce operation was not a success")
###########################################################################
# validate a successful deletion of mapreduce operation result from hdfs
###########################################################################
    def test_hdfs_delete_dir(self):
        o,c= self.master_unit.run("su hdfs -c 'hdfs dfs -rm -r /user/ubuntu/teragenout'")
        if c == 0:
            amulet.raise_status(amulet.PASS, msg="mapreduce result deleted from hdfs cluster")
        else:
            amulet.raise_status(amulet.FAIL, msg="mapreduce was not deleted from hdfs cluster")


if __name__ == '__main__':
    runner = TestDeployment()
    runner.run() 